from flask import Flask, jsonify, abort, request, make_response, url_for
from flask_restful import Resource, Api
from json import dumps
import sys, os, importlib, inspect

def GetCustomer(dreq={}):
    req = request or dreq;
    return str(req)

def default():
    return "Test\n\n";

if __name__ == '__main__':
    default();




#!/usr/local/bin/python

from flask import Flask, jsonify, abort, request, make_response, url_for
from flask_restful import Resource, Api
from json import dumps
import sys, os, importlib, inspect
import wsUtils
import cgi

PyUtilsPATH="../../PyUtils";
sys.path.append(PyUtilsPATH)
CWD=os.getcwd()
ABS_PATH = os.path.abspath(CWD)
sys.path.append(ABS_PATH)

app = Flask(__name__)

def echo():
    tags = [t+"="+request.args[t] for t in request.args ];
    reqs =  str(request.json)
    t = str(tags)  + "\n" + reqs + "\n"
    #print (t)
    return t

def process():
    method = request.args['m'] if 'm' in request.args else 'echo' ;
    ret = ""
    print("Processing method: ", method, CWD, ABS_PATH)
    if (method.find(".") > 0):
        [mod, meth] = method.split('.')
        ret = ("Module:  {}, Method: {}\n".format(mod, meth) )
        try:
            ret = wsUtils.Run(mod, meth);
        except Exception as e:
            ret = "Error " + str(e)
    else:
        if (method != "process" and method in globals()):
            ret = globals()[method]()
        else:
            ret = "Something wrong with method: " + method + "\n"
    return ret;
#--------------------------------------------------------------------------
@app.route('/ws', methods=['POST', 'GET', 'OPTIONS', 'HEAD'])
def ws():
#    for a in request.args: print(a, request.args[a])
    if (len(request.args) <= 0):
        return "Version 1.0\n";
    r = process();
    #print(r)
    return r;

#--------------------------------------------------------------------------
@app.route('/run', methods=['POST', 'GET', 'OPTIONS', 'HEAD'])
def run():
    ret = wsUtils.Run('test', 'GetCustomer');
    return ret;

#--------------------------------------------------------------------------
'''
Help:

flask_ws port
'''
def start():
    port = 8080 if len(sys.argv) < 2 else sys.argv[1]
    print( "Starting at port: ", port)
    app.run(debug=True, host="0.0.0.0", port=port)

if __name__ == '__main__':
    start();



import os
import os.path
import sys
import re;
import json;
import os
import numpy as np
from IPython.display import HTML
import pandas as pd
from datetime import timedelta;
import datetime;
from random import randint
from collections import defaultdict
from pylab import rcParams

import matplotlib.pyplot as plt
import pandas as pd
import glob;
import re;
import platform
import matplotlib

import dateutil;
import json;
import urllib.request;



# matplotlib inline
import matplotlib.pyplot as plt
from numpy import *
from collections import Counter
import numpy as np
import pylab as pl
from matplotlib.colors import ListedColormap
from sklearn import neighbors, datasets, cluster, preprocessing, decomposition
from sklearn.decomposition import PCA
import pandas as pd
from pandas.tools.plotting import scatter_matrix
import numpy.random as random
from mpl_toolkits.mplot3d import Axes3D
import glob
import os
import time
from IPython.display import display
from IPython.display import Image

from scipy	import stats;

from sklearn.metrics import euclidean_distances
from sklearn.neighbors import kneighbors_graph
from sklearn.preprocessing import StandardScaler


# -*- coding: utf-8 -*-
# t- test
def ComputeDegreesOfFreedomFor_t_test(x,y, equal_variance = True):
    n1 = len(x)
    n2 = len(y)
    if equal_variance:
        return n1 + n2 -2;
    s1 = x.var()
    s2 = y.var()
    num = pow((s1/n1 + s2/n2),2)
    den = (pow(s1/n1,2) /(n1 -1)) + (pow(s2/n2,2) /(n2 -1))
    ret = num/den;
    ret = round(ret)
    return ret;

def t_critical(alpha, df, tails=1):
    t_critical_one_tailed=stats.t.ppf(1-alpha, df);
    t_critical_two_tailed=stats.t.ppf(1-alpha/2, df);

    if ( tails == 1):
        return t_critical_one_tailed;
    else:
        return t_critical_two_tailed;
#       
# Returns equal Variance = true if there is no difference in variances
#
#
def Ftest(x,y, alpha = 0.05):
    F = x.var()/ y.var();
    n1 = len(x);  # degrees of freedom
    n2 = len(y);  # degrees of freedom
    cl = 1 - alpha; # Confidence Level
    F_c = stats.f.ppf(cl, n1 -1 , n2 - 1);  # n1-1, n2-1 are degrees of freedom
    p = 1- stats.f.cdf(F, n1 - 1, n2 - 1) 

    equal_variance = True;
    if ( F > F_c ):
        equal_variance = False;

    ostr =  "Equal Variance: " + str(equal_variance) + \
            ", F= "+ str(F) + ", F_critical="+ str(F_c) + ",P=" + str(p);
    return (equal_variance , F, F_c, p, ostr);

#
# Chi square Critical value 
def Chi2Critical(df, alpha = 0.05):
    cl = 1 - alpha; # Confidence Level
    Chi2_c = stats.chi2.ppf(cl, df);
    return Chi2_c;
    
from scipy.stats import norm
def NormProb(x, l,u):
    mu = np.mean(x)
    st = np.std(x)
    print(mu, st)
    minX=min(x)
    maxX=max(x)
    n = norm(mu, st) 
    p = n.cdf(u)- n.cdf(l)
    return p;

    
 



#!/usr/local/bin/python
import io, nbformat, subprocess;
import sys, os, importlib, inspect
from default_ws_config import *
#import wsUtils

import psutil

def kill(cmd0=None, cmd1=None):
    pass;

def findPS(cmd0=None, cmd1=None):
    if (cmd0 is None and cmd1 is None):
        return
    
    for proc in psutil.process_iter():
        try:
            n = proc.name()
            cmd = proc.cmdline()
            if ( (cmd0 and cmd[0].find(cmd0) > 0 ) or
                 (cmd1 and cmd[1].find(cmd1) > 0 )  ):
                print ("Found process ", n, cmd, proc.pid )
                return proc
        except:
            pass

    return None

def bookKeeping():
    global notebooks
    cwd=os.getcwd()
    sys.path.append(cwd)
    if (os.path.islink(sys.argv[0])):
        link = os.readlink(sys.argv[0])
        sys.path.append(os.path.dirname(link))

def deploy():
    global notebooks
    global port

    if ( len(sys.argv) > 1 ):
        print ("Importing from ", sys.argv[1]);
        im = importlib.import_module(sys.argv[1])
        notebooks = im.notebooks
        port = im.port

    merge_notebooks(notebooks, 'wsex.ipynb', False);

def run():
    global port
    proc = findPS(cmd1='jupyter-kernelgateway')
    if ( proc is not None):
        print("Process seems to be running ...")
        return

    OPTS='--KernelGatewayApp.api="kernel_gateway.notebook_http" --KernelGatewayApp.prespawn_count=1'
    PORT='--KernelGatewayApp.port=' + str(port)
    URI='wsex.ipynb'
    URI1='--KernelGatewayApp.seed_uri='+URI

    oss = "jupyter kernelgateway {} {} {}".format(OPTS, PORT, URI1)
    print ("Starting ", oss);
    p = subprocess.run(oss , shell=True)
    #p = subprocess.Popen("./kg.sh" )
    #p.wait()

def merge_notebooks(filenames, outf = None, printO=False):
    merged = None
    for fname in filenames:
        print("Merging ", fname, " ...");
        with io.open(fname, 'r', encoding='utf-8') as f:
            nb = nbformat.read(f, as_version=4)
        if merged is None:
            merged = nb
        else:
            # TODO: add an optional marker between joined notebooks
            # like an horizontal rule, for example, or some other arbitrary
            # (user specified) markdown cell)
            merged.cells.extend(nb.cells)
    if not hasattr(merged.metadata, 'name'):
        merged.metadata.name = ''
    merged.metadata.name += "_merged"

    if (printO):
        print(nbformat.writes(merged))
    if (outf is not None):
        print("Writing to file ", outf, " ...")
        with( io.open(outf, "w", encoding='utf-8')) as mf:
            mf.write(nbformat.writes(merged))


# Command line merge tool
def cmd():
    notebooks = sys.argv[1:]
    if not notebooks:
        print(__doc__, file=sys.stderr)
        sys.exit(1)
        
    merge_notebooks(notebooks)

def test():
    try:
        im = importlib.import_module('myconfig');
        po = im.port
        nb = im.notebooks
    except:
        po = "NONE"
        nbs= "NONE"
        pass;


    print ("Port: " , po)

if __name__ == '__main__':
    if ( len(sys.argv) > 1 and sys.argv[1] == 'test'):
        test()
        exit();
    bookKeeping()
    deploy();
    run();



import numpy as np
from sklearn import preprocessing
from sklearn.decomposition import PCA
import pandas as pd
import os
from IPython.display import display
from IPython.display import HTML
import dateutil;
import json;
import urllib.request;
import re
import base64
from pandas import ExcelFile
from io import StringIO, BytesIO
import xml.etree.ElementTree as ET


import matplotlib
#matplotlib.style.use('ggplot')

np.set_printoptions(precision=2, linewidth=100)
pd.set_option('display.width', 1000)
pd.set_option('precision',5);
pd.set_option('display.precision', 5)
pd.set_option('mode.sim_interactive', True);
pd.set_option('display.max_rows', 9)
pd.set_option('display.float_format', lambda x: '%.3f' % x)

'''
Loads data set for person comparative analysis
Problems and ISSUES - Check the following:
==========================================
** Does your data have headers! If not you need more complex call
If so, Pass "headers=None" and pass names

'''

#Make this generic
class Map(dict):
    """
    Example:
    m = Map({'first_name': 'Eduardo'}, last_name='Pool', age=24, sports=['Soccer'])
    """
    def __init__(self, *args, **kwargs):
        super(Map, self).__init__(*args, **kwargs)
        for arg in args:
            if isinstance(arg, dict):
                for k, v in arg.items():
                    self[k] = v

        if kwargs:
            for k, v in kwargs.iteritems():
                self[k] = v

    def __getattr__(self, attr):
        return self.get(attr)

    def __setattr__(self, key, value):
        self.__setitem__(key, value)

    def __setitem__(self, key, value):
        super(Map, self).__setitem__(key, value)
        self.__dict__.update({key: value})

    def __delattr__(self, item):
        self.__delitem__(item)

    def __delitem__(self, key):
        super(Map, self).__delitem__(key)
        del self.__dict__[key]
        
# LIMITATIONS: Works only if all nodes have same tags
# 1M rows must be good enough
#
def getDFFromXML(file, xmlTag=None, maxRows=1000000):
    with open(file, "r") as f:
        xmlText = f.read()

    root = ET.fromstring(xmlText)
    
    if (xmlTag):
        iters= root.findall(path=xmlTag)
    else:
        iters = root;
    
    cols=[]
    for i,e in enumerate(iters):
        for j,e1 in enumerate(e):
            cols.append(e1.tag)
        break;
    
    rr=[]
    for i,e in enumerate(iters):
        r = []
        for j,e1 in enumerate(e):
             r.append(e1.text)
        rr.append(r)    
        if (i>maxRows):
            break;

    df=pd.DataFrame(rr)
    df.columns = cols
    
    return df;

def DetermineSeperator(line):
    sep = ","    
    split2  = line.split("\t");
    if (len(split2) > 1 ):
        sep = "\t";   
    return sep;
    
def getAuraDF(link):
    f = urllib.request.urlopen(link)
    js = f.read().decode('UTF-8')
    fjs=re.sub('[\n\r|\r\n|\n\s*]+', '\n', js)
    js=re.sub('^\n', '', fjs)
    if (js.find("$rs=") > 0):
        js = js[js.find("$rs=")+5:]
        #print(js[0:100])
        data = json.loads(js)
        df=pd.DataFrame(data['rows'],columns=data['colnames'])
        return df
    else:
        return js

def getFormulas(ws, df2):
    from collections import defaultdict
    formulas= defaultdict(str);

    rows = df2.shape[0]
    cols = df2.shape[1]    
    for i in range(rows):
        for j in range(cols):
            ii = i+1
            jj = j+1
                
            c= ws.cell(row=ii, column=jj);
            v = c.value;
            
            if ( v is not None and type(v) == str and v.strip().startswith('=') ):
                formulas[(ii,jj)] = v
    return formulas

def getExcelFile(fileName, sheetname=0):
    import openpyxl
    df2 = pd.read_excel(fileName, header=None, sheetname=sheetname)
    colNames = [openpyxl.utils.get_column_letter(c) for c in range(1,df2.shape[1]+1) ]
    df2.columns = (colNames)

    df2.fillna('', inplace=True)
    df2.index = range(len(df2))
    wb = openpyxl.load_workbook(fileName)
    if ( type(sheetname) == int):
        sheetname = wb.sheetnames[sheetname]
    ws=wb[sheetname]
    df2.formulas = getFormulas(ws, df2)
    
    return df2;
    
def getDF(fileName, debug=False, headers=None, names=None, usecols=None, checkForDateTime=False, 
          seperator=None, index_col=None, sheetname=0, xmlTag=None):
    
    if (    not (fileName.startswith("http://"))  and
            not (fileName.startswith("https://")) and
            not os.path.exists(fileName)):
        #raise Exception( fileName + " does not exist")
        print ("ERROR: *** " +fileName + " does not exist");
        return None;
    
    sep = seperator or ",";
    df1=None
    if (fileName.startswith("http")):
        df1 = getAuraDF(fileName)
    elif fileName.endswith(".xlsx") or fileName.endswith(".xlsm") or fileName.endswith(".xlsb"): 
        df1 = getExcelFile(fileName, sheetname)
    elif (fileName.endswith(".xml")):
        df1 = getDFFromXML(fileName, xmlTag)
    elif ("/aura/" in fileName):
        df1 = getAuraDF(fileName);
        return df1
    else:
        sep = ","
        if not fileName.endswith(".csv"):
            with open(fileName, 'r') as f:
                line    = f.readline();    
                #split1  = line.split(",");
                sep = DetermineSeperator(line);
                
        df1 = pd.read_csv(fileName, sep=sep, header=headers, low_memory=False,
                       skipinitialspace =True, names=names, comment='#', usecols=usecols)
    return df1;

    
def LoadDataSet(fileOrString, columns=None, excel = False,
                debug=False, headers=0, names=None, checkForDateTime=False, usecols=None, seperator=None,
                index_col=None,sheetname=0, xmlTag=None, **kwargs):
    
    if(type(fileOrString) == bytes and excel ):
        d= base64.decodestring(fileOrString);
        ex = ExcelFile(BytesIO(d) );
        df = ex.parse(ex.sheet_names[-1])
        df = df.fillna('');
        return df;
        
    if (fileOrString.find("\n") >=0 ):
        ps = [line.strip() for line in fileOrString.split('\n')
                if line.strip() != '' and not line.startswith("#") ];
        if (seperator is None):
            sep = DetermineSeperator(ps[0]);
        else:
            sep = seperator;
        ns = [p.split(sep) for p in ps]
        df1 = pd.DataFrame(ns[1:], columns=ns[0], **kwargs);
    else:               
        df1 = getDF(fileOrString, debug=False, headers=headers, names=names, checkForDateTime=checkForDateTime, 
                    usecols=usecols, seperator=seperator, index_col=index_col, sheetname=sheetname, xmlTag=xmlTag)     

    if ( df1 is None or str(type(df1)).find("DataFrame") < 0):
        return df1;
    #df1=df1.convert_objects(convert_numeric=False)
    df2 = df1[columns] if (columns != None ) else df1;

    if (checkForDateTime):
        for i, c in enumerate(df1.columns):
            if (df2.dtypes[i] != object ):
                continue;
            s = df2[c][0]
            if ( len(s) < 8): continue;
            try:
                dateutil.parser.parse(s);
            except:
                continue; 
            print ("Trying to convert to datetime:"+ c);
            df2[c] =  pd.to_datetime(df2[c])  
            
    if debug:
        print ("Printing 5 of %d rows", df2.shape[0]);
        print (df2[:5]);
        
    return df2;


def normalizeData(df):
    df1 = df.select_dtypes(exclude=[object])
    vals = df1.values
    cols = df1.columns

    d = preprocessing.scale(vals)
    df2  = pd.DataFrame(d, columns=cols)
    return df2;





from flask import Flask, jsonify, abort, request, make_response, url_for
from flask_restful import Resource, Api
from json import dumps
import sys, os, importlib, inspect

def loadModule(config_file):
    with open(config_file) as f:
        code = compile(f.read(), config_file, 'exec')
        exec(code, globals(), locals())

def Run(file=None, method=None, reload=False):
    if (file not in sys.modules):
        im = importlib.import_module(file)
    elif(reload):
        im = importlib.reload(file)

    if (file not in sys.modules):
        return ("No File: " + file);

    im = importlib.import_module(file)

    thefunc = getattr(im, method)
    if (thefunc is None):
        return "No Method: " + method 

    return thefunc();

def default():
    ret = "Test\n\n"
    print(ret)
    return ret

cmd_folder = os.path.realpath(os.path.abspath(os.path.split(inspect.getfile( inspect.currentframe() ))[0]))
if cmd_folder not in sys.path:
    sys.path.insert(0, cmd_folder)
print(cmd_folder);

if __name__ == '__main__':
    Run('test', 'default');




import numpy as np
#import pylab as pl
#from matplotlib.colors import ListedColormap
#from sklearn import neighbors, datasets
import matplotlib.pyplot as plt
import pandas as pd
from IPython.display import HTML
import os
import glob;
import re;
#plt.style.use('fivethirtyeight') # Good looking plots
pd.set_option('display.max_columns', None) # Display any number of columns
import platform
import matplotlib
from sklearn import neighbors, datasets, cluster, preprocessing, decomposition
from sklearn.decomposition import PCA
from Features import prepareDF
from matplotlib import colors
import inspect
import re;
import json;
import datetime

if (platform == "Windows"):
    from win32com.client import Dispatch
    from win32com.client.gencache import EnsureDispatch
    from win32com.client import constants
    from IPython.display import IFrame

np.set_printoptions(precision=2, linewidth=1000)

matplotlib.rcParams['lines.linewidth'] = 1
matplotlib.rcParams['lines.color'] = 'r'

#=======================================================================================
def Excel2Html(file, overwrite=True, show=True, leaveItOpen = True, 
                   width="100%", length="400px"):
    f = file.replace("/", "\\");
    #xl = Dispatch('Excel.Application')
    xl = EnsureDispatch ("Excel.Application")
    cwd=os.getcwd() + "\\" + f
    ext = cwd.split(".")[-1]
    nef=cwd.replace(ext, "html")
    nhtml = file.replace(ext, "html")

    fileOpenNow = False;
#    if (not os.path.exists(cwd)):
#        display("File " + cwd + " not found");
#        return;
#    else:
#       try:
#           f = open(cwd, "r+");
#           f.close();
#       except:
#           fileOpenNow =True;
    print (cwd, nef, nhtml, fileOpenNow)
           
    wb=xl.Workbooks.Open(cwd)
    xl.Visible = True #-- optional
    if os.path.exists(nef):
        os.remove(nef)
    wb.SaveAs(nef, constants.xlHtml)
    wb.Close()
#    if (fileOpenNow or leaveItOpen):
    wb=xl.Workbooks.Open(cwd)
    del xl;
    if (show):
        display(IFrame(nhtml, width, length))    

#=======================================================================================
# EXAMPLE USE
# graph a function 
# graphFunction(lambda x: x ** 3, 0,10)
#
# graphFunction('x', 0,1, "r", "$x$", "$z$", "", '.') #, 'o', "sada")
# graphFunction('1/(1+np.exp(-x))', -6,6, "g", "$x$", "$z$", "",'.',"$z=\\frac{1}{1+e^{-\\theta^{T} x}}$") #, 'o', "sada")
# graphFunction(lambda x: log(x), 0,6, "b", "$x$", "$z$", "",'.',"$log(x)$") #, 'o', "sada")

def graphFunction(formula, xmin, xmax, c=None, xlabel= None, ylabel=None, title=None, marker=None, 
                    label=None, legend=True, legendLoc=2):
    x = np.linspace(xmin, xmax, 100)
    if ( callable(formula)):
        y = np.apply_along_axis(formula, 0, x)
        #print ("Evaluating Function", str(formula), y)
    elif (type(formula) == str):
        y = eval(formula)
        #print ("Evaluating", formula, y)
    else:
        y = formula

    #return 
    label = label if label else str(formula);
    plt.plot(x, y, c=c, marker=marker, label=label, linewidth=1)
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.title (title, fontsize = 15)
    fs = 20 if (label.find("$") > 0) else 15
    plt.legend(fontsize=15, loc=legendLoc) if ( legend) else None;

'''
=======================================================================================
Convenient class A to manage and display matrices

# Example Usage
a= A(('1 7 2 3. 5 5 6 6 7 8.' * 3 +";") * 4, "A")
#a.a = a.a.T # create transpose
a.l          # display latex version of the matrix
=======================================================================================
'''
from IPython.display import display, Math, Latex
import numpy as np
import re
class LA:
    print_width = 120;

    def dtype(self):
        return "A"

    def __init__(self, s = "", name ="\_"):
        self.type = "ClassA"

        if ( type(s) is np.ndarray or type(s) is np.matrix):
            self.a = np.matrix(s)
        #if ( type(s) is np.ndarray or type(s) is np.matrix):

        if ( type(s) is str):
            self.a = A.setM(s)
        elif ( type(s) is list ):
            self.a = np.array(s) #[None]
        self.name = name;

    @staticmethod
    def setM( str = None):
        str = str.replace("[", "")
        str = str.replace("]", "")
        ss = str.split(';')
        r = None;
        for i,s in enumerate(ss):
            s = s.strip();
            if ( len(s) <= 0):
                continue;
            a = A.setA(s)
            r = a if i == 0 else np.vstack((r,a))

        return np.matrix(r)

    @staticmethod
    def setA(s = None):
        if (s == None):
            return []
        s = s.replace(",", " ")
        a = s.split()
        try:
            a = list(map(int, a))
        except:
            a = list(map(float, a))

        a = np.array(a)
        return a

    def p(self):
        np.set_printoptions(precision=2, linewidth=180)
        print ("shape=", self.a.shape, " name:", self.name)
        s = str(self.a)
        s = s.replace('[', '')
        s = s.replace(']', '')

        s =  "[" + s + "]"
        return s

    def pr(self,output=True, noName=False):
        np.set_printoptions(precision=2, linewidth=180)
        m = self.a;

        dim = self.name + "_{" + " \\times ".join(map(str, (m.shape) )) + "} = ";
        lhs = "" if noName  else dim;

        s = str(m)
        s = s.replace('[', '')
        s = s.replace(']', '')
        s = s.replace('\n', '\\\\\\\\<NEW-LINE>')
        s = re.sub( '\s+', ' ', s ).strip()
        s = s.replace('<NEW-LINE>', "\n")
        s = re.sub('\n\s+', '', s)
        s = s.replace(' ', ' & ')
        s = lhs + "\\begin{bmatrix}\n" + s + "\n\\end{bmatrix}"
        #print self.a
        if ( output):
            display(Math(s))
        return s;

    @staticmethod
    def M(m, name="", call_display=True, showdim=True, precision=4):
        np.set_printoptions(precision=precision, linewidth=180)
        name = name + " =" if name != "" else ""
        dim = "";
        if (showdim):
            dim = " \\times ".join(map(str, (m.shape) )) ;
        s = str(m)
        s = s.replace('[', '')
        s = s.replace(']', '')
        s = s.replace('\n', '\\\\\\\\<NEW-LINE>')
        s = re.sub( '\s+', ' ', s ).strip()
        s = s.replace('<NEW-LINE>', "\n")
        s = re.sub('\n\s+', '', s)
        s = s.replace(' ', ' & ')
        s = name + "\\begin{bmatrix}\n" + s + "\n\\end{bmatrix}" +  dim + "\n"
        #print self.a
        if ( call_display):
            display(Math(s))
        return s;


    #Display thr matrix
    @staticmethod
    def display(*M):
        s = ""
        for m in M:
            s+= LA.M(m, call_display=False, showdim=False);
        display(Math(s))

    def d(self):
        self.display(self.a, self.name)
        return self.a

    # Returns Latex information
    @property
    def T(self):
        return A(self.a.T)

    # Returns Latex information
    @property
    def l(self, output=True):
        np.set_printoptions(precision=2, linewidth=180)
        name = self.name + " =" if self.name != "" else ""
        dim = " \\times ".join(map(str, (self.a.shape) )) ;
        s = str(self.a)
        s = s.replace('[', '')
        s = s.replace(']', '')
        s = s.replace('\n', '\\\\\\\\<NEW-LINE>')
        s = re.sub( '\s+', ' ', s ).strip()
        s = s.replace('<NEW-LINE>', "\n")
        s = re.sub('\n\s+', '', s)
        s = s.replace(' ', ' & ')
        s = name + "\\begin{bmatrix}\n" + s + "\n\\end{bmatrix}" +  dim + "\n"
        #print self.a
        #if (output): display(Math(s))
        return s;

    def __str__(self):
        return self.p()

    def __add__(self, o):
        if (type(o) is int):
            r = self.a + 3
        else:
            r = np.add(self.a, o.a)
        ret = A(r)
        return ret


    def __mul__(self, o):
        if (type(o) == A ):
            r = self.a * o.a
        else:
            r = self.a * o.a
        ret = A(r)
        return ret

def formatContentDELETEIT(c):       
    c1 = str(c).lower().strip()
    g=[k.lower().strip() for k in "complete, finished, success, Yes".split(",") ]
    r=[k.lower().strip() for k in "error, err, failed, no".split(",") ]
    y=[k.lower().strip() for k in "pending, ongoing, current".split(",") ]
    s = ""
    if (c1 in g):
         s = "bgcolor=lightgreen";
    elif (c1 in r):
         s = "bgcolor=#FFAEAE";
    elif (c1 in y):
         s = "bgcolor=lightyellow";
        
    return "<td " + s + ">" + str(c) + "</td>"
    
#if not os.path.exists("temp"):
#    os.mkdir("temp")

#Lets Clean up before we start
[os.unlink(f) for f in glob.glob("./temp/*.png")]
    
def PCAPlot(dfL, predictColumn, s =10):
    predictColumnIdx = predictColumn+'_idx'
    
    ny = dfL[predictColumn]
    df = prepareDF(dfL, makeCopy=True)
    df = df.drop(predictColumn, axis=1)
    pca= PCA(n_components= 2)
    pca.fit(df)
    nX = pca.transform(df)
    
    le = preprocessing.LabelEncoder()
    labels = le.fit_transform(ny)
    le.classes_
    
    nDf = pd.DataFrame(nX)
    nDf[predictColumn] = ny;
    nDf[predictColumnIdx] = labels;
    
    c="r,g,b,c,m,y,k,w".split(",")
    
    for i,j in enumerate(le.classes_):
        dd = nDf[nDf[predictColumnIdx] == i]
        ll = str(le.classes_[i]);
        lb = ll + ":" + str(i) if ( ll != str(i)) else str(ll);
        #print( i,j )
        #plt.scatter(dd[[0]], dd[[1]], s=40, c=dd[predictColumnIdx].apply(lambda x:c[x]));
        plt.scatter(dd[[0]], dd[[1]], s=40, c=dd[predictColumnIdx].apply(lambda x:c[x%len(c)]), label=lb);
    
    plt.legend();

    return nDf;


#
# x must be in percentages of use - otherwise convert to percentages before this call
#
def plotPercentHist(x, bins=10, rangeI=(0.0,1.0)):
    h, be = np.histogram(x,bins=bins,range=rangeI, normed=True)
    plt.bar(be[:-1],h*100,width=be[1])
    plt.xticks(be);
    return h,be

#def pltBar(x, labels=None, bottom=None):
   #x = x.value_counts()
#plt.bar(x, range(0, len(x)), bottom=bottom);
    #if (labels is None):
    #labels = range(0,len(x));
    #plt.xticks(labels);

import math
import StatUtils
def plthist(x, y=None, bins='auto', alpha=0.75, title=None, grid = True, xlabel=None, 
            ylabel=None, label=None, axis=None, subplot=None, 
            facecolor=None, ablines=[], legend=False,
            low=None, high=None,  
           ):
    c='brygcmykw';
    if(subplot):
        plt.subplot(subplot)
        
    N, bins, patches = plt.hist(x, bins=bins, alpha=alpha, label=label, facecolor=facecolor)

    if (low is not None and high is None ):
        high = math.inf
    if (high is not None and low is None):
        low = -math.inf
    if(low is not None and high is not None ):
        ca=plt.gca()
        w = min(high, max(x)) - max(low, min(x));
        l = max(low, min(x))
        
        hm = max(N)/4
        rect = plt.Rectangle( (l,0), w,hm, alpha=.6, facecolor='y')
        ca.add_patch(rect)
        
        rx, ry = rect.get_xy()
        cx = rx + rect.get_width()/2.0
        cy = ry + rect.get_height()/2.0
        r = StatUtils.NormProb(x, low,high) * 100
        r1 = "Prob: {0:0.0f}-{1:0.0f} : {2:0.2f}%".format(low, high,r)
        plt.annotate(str(r1), (cx, cy), color='w', weight='bold', fontsize=8, ha='center', va='center')
        
    #print(low, high)    
        
        
    if(grid is not None): plt.grid(grid)
    if(xlabel is not None): plt.xlabel(xlabel)
    if(ylabel is not None): plt.ylabel(ylabel)
    if(axis is not None):  plt.axis(axis)
    if(title is not None): plt.title(title)
    
    for i,f in enumerate(ablines):
        try:
            methodToCall = getattr(x, f)
            v = methodToCall();
            #plt.axvline(v, color='b', linestyle='dashed', linewidth=2, )
            lab = f + " : " + "{0:0.2f}".format(v)
            lab = f
            plt.axvline(v,  color=c[i%len(c)], linestyle='dashed', linewidth=1, label=lab)
            #plt.text(v+.1, 75, lab ,rotation=90)
            #plt.annotate(lab, xy=(v, 1), xytext=(v, 75),arrowprops=dict(facecolor='black', shrink=0.05),)
            
        except:
            print("Method '{}' not found".format(f))
            
    if (legend):
        leg=plt.legend(loc='best', frameon=1)    

'''
=======================================================================================
Print analysis on the Pandas dataframe
=======================================================================================
'''
def searchDF(df, s="", cols=[], maxRows=10):
    rows=[]    
    for i, r in df.iterrows():
        for j, c in r.iteritems():
            if (len(cols) > 0 and not c in cols):
                break;
            if ( re.match(s, str(c), flags=re.M|re.DOTALL|re.I) ):
                maxRows = maxRows -1;
                rows.append(i)
                break;
        if (maxRows ==0):
            break
    df1 = df.iloc[rows];
    return df1;
    

def colTypesDF(df):
    dfTypes=pd.DataFrame(df.dtypes)
    dfTypes=dfTypes.transpose()
    cols=[];
    for i, c in enumerate(dfTypes.columns):
        nc = (str(c) + "\n\t(" + str(dfTypes.iloc[0][i])+")")
        cols.append(nc);
    return cols;

def getFileName(df,idx):
    c = df.columns[idx];
    c=re.sub(r'[\s+?\']', '', str(c))
    prefix = hex(id(df)) + "-"+c;
    figName1 = "temp/"+ prefix + ".png";    
    
    if not os.path.exists("temp"):
        os.mkdir("temp")
    
    return figName1;    

def isAnyDataPoint(t):  
    for k in t:
        if not np.isnan(k):
            return True;
    return False;
    
def createIcon(df,idx):
    t = df.iloc[:,idx]

    scale=7;
    figName1 = getFileName(df,idx)

    if (os.path.exists(figName1)):
        return figName1;

    k = "hist"
    if (df.dtypes[idx] == object or str(df.dtypes[idx]).find("date") >= 0 ):
        u=len(t.unique());
        if (u>100):
            return None;
        else:
            t = t.value_counts();
            k = "bar";
    if not isAnyDataPoint(t):  
       return None;
               
    #print ("N= " , df.columns[idx]);
    ax=t.plot(kind=k, figsize=(1*scale, 0.5*scale), grid=True);
    ax.get_xaxis().set_visible(True)
    ax.get_yaxis().set_visible(True)
    ax.set_frame_on(False)
    fig = ax.get_figure()
    fig.savefig(figName1,  transparent=True);
        
    plt.close();
        
    return figName1;


def getDesc(df, idx):
    t = df.iloc[:,idx].describe(include='all')

    out = "";
    for ii,jj in enumerate(t):
        if ( str(type(jj)).find('float') >=0):
           val = "{0:0.2f}".format(jj) 
        else:
           val = str(jj)
        tval = "<td>{} : {} </td>".format(t.index[ii] , val)
        
        #if (df.dtypes[idx] == object or str(df.dtypes[idx]).find("date") >= 0 ):
        if (ii == 4):
            out += "</tr><tr>"+tval
        else:
            out += tval;

    out = "<table><tr>" + out + "</tr></table>";
    return out


def getIcons(df,h):
    h1="<tr><td style='text-align: ceter;'></td >";
    idx=0;
    for i, c in enumerate(df.columns):
        fig = None 
        try:
           fig = createIcon(df,idx);
        except:
           print ("Error while getting icon for ", c);
        if ( fig is None):
            h1 = h1 + "<td align=center></td>";
        else:
            
            #fig = "/files/" + fig;
            desc = getDesc(df, idx);
            h1 = h1 + '''
<td align=center>{}<br/><a class='thumbnail' href='#thumb'>
<img src='{}' border=0 style='{{margins: 0;}}' width=64 height=64 /> 
<span><img src='{}' /><br />
{} {}
</span></a></td>'''.format(str(df.dtypes[i]),fig, fig, '', desc); #getDesc(df,idx);

            #h1 = h1 + "<td><a class='thumbnail' href='#thumb'><img src='" + fig;
            #h1 = h1 + "' border=0 style='{margins: 0;}' width=64 height=64 ";
            #h1 = h1 + "/> <span><img src='"+ fig + "' /><br /></span></a>";
            #h1 = h1 +"onmouseover='this.width=500;' onmouseout='this.width=64' >" 
            #h1 = h1 + "</td>";  
            
        idx=idx + 1;          
    h1 = h1 + "</tr>\n";
    idx = h.find("<tr");
    ret = h[:idx] +h1+h[idx:];
    return ret;    

def addDescribe(df,h):
    df1d= df.describe(include='all')
    hd = df1d.to_html();
    idx1 = hd.find("<tbody>") + 8;
    idx2 = hd.find("</tbody>");
    rep = hd[idx1:idx2] 
    rep = rep.replace("<tr>", "<tr bgcolor=#e6e6fa>",4)
    rep = rep.replace("<tr>", "<tr bgcolor=#dddddd>")
    rep = rep.replace("nan", "-")
    rep = rep.replace("NaN", "-")
    idx = h.find("<tr");
    ret = h[:idx] +rep+h[idx:];
    ret = ret.replace("<th>", "<th bgcolor=#e6e6fa>",4)
    ret = ret.replace("<th>", "<th bgcolor=#6495ed>")
   
    return ret;

def getHTMLTableRows(dff, startRow=0, maxRows = 5):
    
    startRow = startRow if startRow >=0 else 0;
    e = startRow + maxRows
    
    if (e > len(dff)):
        ddd=dff[-maxRows:]
    else:
        ddd=dff[startRow:e]

    vals = re.findall('<td>(.*?)</td>',  ddd.to_html(), flags=re.DOTALL|re.M|re.MULTILINE|re.IGNORECASE)
    vals = [v.replace("'", '\\\'') for v in vals]
    vals = [v.replace("\n", '\\n') for v in vals]
    
    if (dff.index.dtype_str.startswith('datetime')):
        idxs = list(ddd.index.astype(str))
    else:
        idxs = [""] + (list(ddd.index) )
    
    #ret = '''"vals": {}; "idxs": {}'''.format(json.dumps(vals), json.dumps(idxs))
    ret = '''{{"vals": {}, "idxs": {}}}'''.format(json.dumps(vals), (idxs) )
    
    ret = ret.replace("'", '"')
    
    return ret

def getHTMLTableRowsFromIndex(dff, idx=0, dispRows = 5, goBack=True):
    
    if (dff.index.is_numeric()):
        startRow = dff.index.get_loc(int(idx) )
    else:
        startRow = dff.index.get_loc(idx )

    if (goBack):
        topRow = startRow - dispRows;
        startRow = startRow - 2*dispRows +2
        if ( topRow < 0):
            return""; #Already showing top row
    else:
        if ( startRow >= len(dff)-1):
            return ""; # already showing last Row
    startRow = max(startRow,0)
    e = startRow + dispRows
    ddd=dff[startRow:e]

    vals = re.findall('<td>(.*?)</td>',  ddd.to_html(), flags=re.DOTALL|re.M|re.MULTILINE|re.IGNORECASE)
    vals = [v.replace("'", '\\\'') for v in vals]
    vals = [v.replace("\n", '\\n') for v in vals]
   
    if (dff.index.dtype_str.startswith('datetime')):
        idxs = list(ddd.index.astype(str))
    else:
        idxs = [""] + (list(ddd.index) )
    
    #ret = '''"vals": {}; "idxs": {}'''.format(json.dumps(vals), json.dumps(idxs))
    ret = '''{{"vals": {}, "idxs": {}}}'''.format(json.dumps(vals), (idxs) )
    
    ret = ret.replace("'", '"')
    
    return ret

def getHTMLTableRowsFromSearch(dff, searchString=None, maxRows=10):
    searchString = str(searchString)
    if (searchString is None or len(str(searchString).strip()) < 0):
        return '';
    
    ddd = searchDF(dff, s=searchString, maxRows=maxRows)
    
    if(len(ddd) <= 0 ):
        return ''    
    return ddd.to_html()


def UpdateDataFrame(dff, row, col, val):
    if(len(dff) < 0):
        return '';
    
    col = int(col)
    try:
        newVal = pd.Series(val).astype(dff.dtypes[col])[0]
        oldVal = val
        
        if (dff.index.is_numeric()):
            oldVal = dff.ix[int(row), col]
            dff.ix[int(row), col] = newVal
        else:
            oldVal = dff.ix[row, col]
            dff.ix[row, col] = newVal

        if (oldVal != newVal):
            print("old: {} , new: {}".format( oldVal, val))
    except:
        pass;
    
    return [oldVal, newVal]


def UpdateDataFrameFromHTML(html, dff=None):

    vals = re.findall('<td.*?>(.*?)</td>',  html, flags=re.DOTALL|re.M|re.MULTILINE|re.IGNORECASE)
    idxs = re.findall('<th.*?>(.*?)</th>',  html, flags=re.DOTALL|re.M|re.MULTILINE|re.IGNORECASE)
    rows = re.findall('<tr.*?>(.*?)</tr>',  html, flags=re.DOTALL|re.M|re.MULTILINE|re.IGNORECASE)


    nRows = len(rows)
    nCols = int(len(vals)/nRows)
    
    #console.log ("===> Reshaping", nRows, nCols);
    vals = np.array(vals).reshape(nRows, nCols)

    #print( vals, len(vals), nCols, nRows)

    dft = pd.DataFrame(vals)
    dft.index = idxs
    if (dff is None ):
        return dft
    
    dft.columns= dff.columns;
    dft.index  = dft.index.astype(dff.index)

    for i,dt in enumerate(dft.dtypes):
        if ( dt != dff.dtypes[i]):
            dft[dft.columns[i]] = dft[dft.columns[i]].astype(dff.dtypes[i])

    for i, r in dft.iterrows():
        dff.loc[i:i,:] = dft.loc[i:i,:]

navigation_buttons = '''
<div style="display:block;height:20px">

<input id='<TABLE_ID>_goto' type=text value=''  size=4 
onkeyup = "if (event.keyCode == 13) TableShowRows( $(this).val(), MAXDISP, '#<TABLE_ID>')">

<input type=button value='<<' onclick="Page( 1, MAXDISP, '#<TABLE_ID>' );">
<input type=button value='>>' onclick="Page( 0, MAXDISP, '#<TABLE_ID>' );">

<input type=button value='Save' onclick="SaveDataFrameFromHTML('#<TABLE_ID>', 'DFF_PY_VAR_<TABLE_ID>');">
<input type=button value='Search>' onclick="SearchDataFrame('DFF_PY_VAR_<TABLE_ID>', '#<TABLE_ID>', MAXDISP);">
<input id='<TABLE_ID>_search' type=text value=''" size=10
    title="RE search support: ex: ^13$\\b to search for 13"
    onkeyup = "if (event.keyCode == 13) SearchDataFrame('DFF_PY_VAR_<TABLE_ID>', '#<TABLE_ID>', MAXDISP);">
</div>
<script>
AddFocus('#<TABLE_ID>', MAXDISP);
$("#<TABLE_ID> th").resizable()

</script>
''';

def displayDFs(dfs, maxrows = 6, startrow=0, showTypes = False, showIcons=True, tableID=None, showNav= True,
               search=None, cols=[],  showStats = False, editable=True, useMyStyle=True, donotDisplay=False ):
                   
    if ( type(dfs) !=list and type(dfs) != tuple):
        dfs = [dfs];
        
    otr = "<table style='vertical-align: top;'>" if (len(dfs) >1) else "<table wwidth=100% style='vertical-align: top;'><tr>"
    bg1="#ffffff";
    bg2="lightblue";
    bg = bg2;
    for i, nd in enumerate(dfs):
        if ( nd is None  ):
            otr += "<td> None</td><td>&nbsp;</td>"
            continue;
            
        bg = bg2 if ( bg == bg1 ) else bg1;
        dim = str(nd.shape[0]) + " rows x " + str(nd.shape[1]) + " columns";
        
        if(search):
            d =searchDF(nd,search,cols);
        else:
            if( startrow+maxrows > len(nd) ):
                d = nd[-maxrows:] 
            else:
                d = nd[startrow:startrow+maxrows] 
            
        if (showTypes):
            cols=colTypesDF(d);
            d.columns = cols
        h = d.to_html();
        #
        if(len(dfs) == 1):
            h = h.replace("<table ", "<table wwidth=100% ")
        #
        if (editable):
            h = h.replace("<td", "<td contenteditable ")
        #
        if (useMyStyle):
            h = h.replace("class='dataframe'", "class='ourTableStyle' ")
            h = h.replace('class="dataframe"', "class='ourTableStyle' ")
            h = h.replace('<table ', "<table cellpadding=0 cellspacing=0 ")
            #h = h.replace("border=", "border=")
        
            
        
        shIcons = showIcons; 
        if ( type(showIcons) ==list and type(showIcons) != tuple):
            shIcons = showIcons[i];

        if (shIcons and nd.shape[0] > 0):
            h = getIcons(nd,h);
        if (showStats and nd.shape[0] > 0):
            pass; #h = addDescribe(nd,h);
            #NOt doing this anymore - too ugly
       
    
            
        if(showNav):
            dttm = str(int(datetime.datetime.now().timestamp()*1000) )
            tableID = "tableID_" + dttm 
            nd.tableID = tableID
            
            h = h.replace("<table ", "<table id='{}' ".format(tableID) )

            dfVarNme = 'DFF_PY_VAR_'+tableID;
            inspect.stack()[1][0].f_globals[dfVarNme] = nd

            op = '''<div id='tab_<TABLE_ID>' style='ddisplay:none;width:100%' >
            {}
            <div id='<TABLE_ID>_searchResults'></div>
            </div>'''.format(h + navigation_buttons)
            op = op.replace("<TABLE_ID>", tableID).replace('NUMROWS', str(len(nd))).replace('MAXDISP', str(maxrows) )
            h = op
            
        else:
            if(tableID):
                h = h.replace("<table ", "<table id='{}' ".format(tableID) )

    
        
        tabSep = "<td>&nbsp;</td>" if len(dfs) > 1 else "";
        otr += "<td style='text-align:left;' bgcolor=" + bg + ">" + dim + " var: " + dfVarNme + "<br>\n" + h + "</td>{}".format(tabSep)
    otr += "</tr></table>"
    if (not donotDisplay):
        display(HTML(otr))
        
    return otr

